///////////////////////////////////////////////////////////////////////////////
// DbgPrnHk.h

#ifndef __DEBUG_PRINT_HOOK_H__
#define __DEBUG_PRINT_HOOK_H__

extern "C" {

#pragma pack(push, 1)

#ifndef USER_MODE

#pragma pack(push, 8)

#include <ntddk.h>                  // various NT definitions
#include <stddef.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include "ntddk_ex.h"
#include "tools.h"

#pragma pack(pop)

#ifdef POOL_TAGGING
#ifdef ExAllocatePool
#undef ExAllocatePool
#endif //ExAllocatePool
#define ExAllocatePool(a,b) ExAllocatePoolWithTag(a,b,'kHPD')
#endif //POOL_TAGGING

#define NT_DBGPRNHK_REG_PATH        L"\\REGISTRY\\MACHINE\\SYSTEM\\CurrentControlSet\\Services\\DbgPrnHk"

#else //USER_MODE

#define NT_DBGPRNHK_REG_PATH        L"SYSTEM\\CurrentControlSet\\Services\\DbgPrnHk"
#define PAGE_SIZE 4096

#endif //USER_MODE

#define NT_DBGPRNHK_REG_TAG_VAL              L"Tag"
#define NT_DBGPRNHK_DRV_TAG                  0x0000DBC0
#define NT_DBGPRNHK_REG_GROUP_VAL            L"Group"

// Device names
#define NT_DbgPrnHk_DOS_NAME     ("\\DosDevices\\DbgPrnHk")
#define NT_DbgPrnHk_NAME         ("\\Device\\DbgPrnHk")
#define NT_DbgPrnHk_NAME_W       (L"\\Device\\DbgPrnHk")
#define NT_DbgPrnHk_USER_NAME    ("\\\\.\\DbgPrnHk")
#define NT_DbgPrnHk_USER_NAME_W  (L"\\\\.\\DbgPrnHk")
#define NT_DbgPrnHk_SVC_NAME     ("DbgPrnHk")

#ifndef CTL_CODE
#include "winioctl.h"
#endif //CTL_CODE

#include "version.h"

#define DPHK_CTL_CODE_X(a,b)    CTL_CODE(FILE_DEVICE_UNKNOWN, a,b, FILE_ANY_ACCESS )

#define IOCTL_DbgPrnHk_GetMessages           DPHK_CTL_CODE_X(0x801, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_SetMessageBufferSize  DPHK_CTL_CODE_X(0x802, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_GetVersion            DPHK_CTL_CODE_X(0x803, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_PostMessage           DPHK_CTL_CODE_X(0x804, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_GetRdtscCalibration   DPHK_CTL_CODE_X(0x805, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_GetRdtsc              DPHK_CTL_CODE_X(0x806, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_CanUnload             DPHK_CTL_CODE_X(0x807, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_PostMessageEx         DPHK_CTL_CODE_X(0x808, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_PostBinMessage        DPHK_CTL_CODE_X(0x809, METHOD_BUFFERED)

#define MAX_MSG_SIZE  (128-sizeof(LARGE_INTEGER)-sizeof(PVOID)*2-sizeof(USHORT)-sizeof(UCHAR)*2-sizeof(UCHAR)*4)

#define TimeStampType_SysPerfCounter   ((ULONG)0x00)
#define TimeStampType_RdtscPerfCounter ((ULONG)0x01)
#define TimeStampType_SysTime          ((ULONG)0x02)
#define TimeStampType_Max              TimeStampType_SysTime

#define DbgCallerMode_K                ((ULONG)0x00)
#define DbgCallerMode_U                ((ULONG)0x01)

#define MsgFlags_DbgCallerMode_K       (DbgCallerMode_K << 2)
#define MsgFlags_DbgCallerMode_U       (DbgCallerMode_U << 2)

#define DbgDataType_Text               ((ULONG)0x00)
#define DbgDataType_Bin                ((ULONG)0x01)
#define DbgDataType_Max                DbgDataType_Bin

#define MsgFlags_DbgDataType_Text      (DbgDataType_Text << (2+2))
#define MsgFlags_DbgDataType_Bin       (DbgDataType_Bin  << (2+2))

#define MsgFlags_ValidMask             (((ULONG)0x1 << (2+2+2))-1)

typedef union _DbgPrnHk_MsgFlags {
    ULONG Flags;
    struct {
        ULONG         TimeStampType:2;
        ULONG         CallerMode:2;
        ULONG         DataType:2;
        ULONG         Reserved0:2;
        ULONG         Reserved:24;
    };
} DbgPrnHk_MsgFlags, *PDbgPrnHk_MsgFlags;

typedef struct _DbgPrnHk_GetMessages_USER_OUT {
    LARGE_INTEGER TimeStamp;
    PVOID         ProcessId;
    PVOID         ThreadId;
    USHORT        Length;
    UCHAR         Irql;
    UCHAR         CpuNumber;
    //
    union {
        ULONG Flags;
        struct {
            ULONG         TimeStampType:2;
            ULONG         CallerMode:2;
            ULONG         DataType:2;
            ULONG         Reserved0:2;
            ULONG         Reserved:24;
        };
    };
    /*
    UCHAR         TimeStampType:2;
    UCHAR         CallerMode:1;
    UCHAR         DataType:2;
    UCHAR         Reserved0:3;
    UCHAR         Reserved[3];
    //
    */
    CHAR          Msg[MAX_MSG_SIZE];
} DbgPrnHk_GetMessages_USER_OUT, *PDbgPrnHk_GetMessages_USER_OUT;

typedef struct _DbgPrnHk_SetMessageBuffer_USER_IN {
    ULONG BufferSize;
} DbgPrnHk_SetMessageBuffer_USER_IN, *PDbgPrnHk_SetMessageBuffer_USER_IN;

typedef struct _DbgPrnHk_GetVersion_USER_OUT {
    ULONG         Major;
    ULONG         Minor;
    ULONG         Sub;
} DbgPrnHk_GetVersion_USER_OUT, *PDbgPrnHk_GetVersion_USER_OUT;

typedef struct _DbgPrnHk_GetRdtscCalibration_USER_IN {
    BOOLEAN       Recalibrate;
    BOOLEAN       ChangeType;
    UCHAR         TimeStampType;  // valid if ChangeType == TRUE
} DbgPrnHk_GetRdtscCalibration_USER_IN, *PDbgPrnHk_GetRdtscCalibration_USER_IN;

typedef struct _DbgPrnHk_GetRdtscCalibration_USER_OUT {
    __int64       RdtscTimeStampCalibration0;
    __int64       SysTimeStampCalibration0;
    __int64       RdtscTimeStampCalibration;
    __int64       SysTimeStampCalibration;
    UCHAR         TimeStampType;
} DbgPrnHk_GetRdtscCalibration_USER_OUT, *PDbgPrnHk_GetRdtscCalibration_USER_OUT;

typedef DbgPrnHk_GetMessages_USER_OUT  DbgPrnHk_PostMessageEx_USER_IN;
typedef PDbgPrnHk_GetMessages_USER_OUT PDbgPrnHk_PostMessageEx_USER_IN;

#ifndef USER_MODE

// Function declarations
extern NTSTATUS DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath);
extern NTSTATUS DbgPrnHkCreateClose(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp);
extern NTSTATUS DbgPrnHkDeviceControl(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp);
extern VOID     DbgPrnHkUnload(IN PDRIVER_OBJECT DriverObject);

#endif //USER_MODE

/* driver registry options */

#define NT_DBGPRNHK_REG_CHECK_IRQL_VAL       L"CheckIrql"
#define NT_DBGPRNHK_REG_BUFFER_SIZE_VAL      L"BufferSize"
#define NT_DBGPRNHK_REG_BLOCK_MSG_VAL        L"DoNotPassMessagesDown"
#define NT_DBGPRNHK_REG_STOP_OVERFLOW_VAL    L"StopOnBufferOverflow"
#define NT_DBGPRNHK_REG_TSTAMP_TYPE_VAL      L"TimeStampType"
#define NT_DBGPRNHK_REG_AGGREGATE_MSG_VAL    L"AggregateMessages"

/*
typedef struct _DBGPRINT_HOOK_OPTIONS {
    PWCHAR OptionName;
    ULONG  OptionType;
    ULONG  UpdateFlag;
    union {
        ULONG OptDword;
        PWCHAR OptStr;
    } DefValue;
    union {
        ULONG OptDword;
        PWCHAR OptStr;
    } Value;
} DBGPRINT_HOOK_OPTIONS, *PDBGPRINT_HOOK_OPTIONS;
*/
//extern DBGPRINT_HOOK_OPTIONS DbgPrintHook_Options[];

typedef struct _DBGPRNHK_INTERNAL_STATE {

    DbgPrnHk_GetVersion_USER_OUT   Ver;

    PDbgPrnHk_GetMessages_USER_OUT MsgBuffer;
    LARGE_INTEGER ReadPosition;
    LARGE_INTEGER WritePosition;
    ULONG   BufferSize;
    ULONG   BufferSizeMask;
    ULONG   QueueSize;
    ULONG   MaxQueueSize;
    BOOLEAN CheckIrql;
    BOOLEAN NoPassMsgDown;
    BOOLEAN StopOnBufferOverflow;
    BOOLEAN LoggingPaused;
    UCHAR TimeStampType;
    BOOLEAN BugCheckRegistered;
    BOOLEAN AggregateMessages;
    UCHAR   Reserved[1]; // padding
    ULONG   MaxPage;

} DBGPRNHK_INTERNAL_STATE, *PDBGPRNHK_INTERNAL_STATE;

typedef union _DBGPRNHK_SIGNATURE_PAGE {
    UCHAR    _Bytes_[PAGE_SIZE];
    struct {
        UCHAR                    Signature[PAGE_SIZE/2];
        PDBGPRNHK_INTERNAL_STATE st;
    };
} DBGPRNHK_SIGNATURE_PAGE, *PDBGPRNHK_SIGNATURE_PAGE;

#define DBGPRNHK_SIGNATURE  "*SIG:DbgPrnHk*\n"

#pragma pack(pop)

};

#endif // __DEBUG_PRINT_HOOK_H__

